/**
 * SearchForm.java 
 * 
 * This class represent the user interface when the user clicks the 'Search'
 * button in the left frame. It extends from Panel and implements an
 * ActionListener to trap the button events generated in this class. The Connection
 * class is used to communicate to the server.
 * 
 * @author xjqiu@npac.syr.edu
 */


import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class SearchForm extends Panel  // class definition
          implements ActionListener {

 GridBagLayout gb = new GridBagLayout();  //use a GridBagLayout
 GridBagConstraints gbc = new GridBagConstraints();  // constraints for the layout

 Color bg = new Color(255,255,255); //background and foreground colors
 Color fg = new Color(0,80,80);
 private Color buttonbg = new Color(166,166,166); //background color
 private Color buttonfg = new Color(3,61,52); //foreground color
 private TextField course; // text boxes for fields
 private Button submit,cancel; // buttons for submit and cancel
 private Label l1, l2; // labels for the textboxes and buttons
 private DayList day; //componenets for DayList
 private TimeList time; //components for TimeList
 private Button list;
 private StatusWin swin; // status window for displaying the status of communication


/**
 * Constructor for the class. Builds up the user interface by placing the componenets
 * in the panel according to the constraints.
 */

 public SearchForm() { 

     setLayout(gb); //sets layout, background color
     setBackground(bg);

     gbc.fill = GridBagConstraints.NONE; // sets fill as none
     gbc.anchor = GridBagConstraints.NORTHWEST; // sets anchor as NORTHWEST

     l1 = makeLabel(" Dept/Course/Section : "); //adds text boxes
     addComp(l1,4,0,1,1,new Insets(-215,0,1,1));
     course = new TextField(15);
     addComp(course,9,5,1,1,new Insets(-215,2,1,1));

     submit = new Button("Submit"); //add submit and cancel and their action listeners
     submit.setBackground(buttonbg);     // which is this class itself
     submit.setForeground(buttonfg);
     submit.setFont(new Font("Helvetica", Font.BOLD, 14));
     submit.addActionListener(this);
     addComp(submit,4,3,1,2,new Insets(-175,0,1,1));
     cancel = new Button("Cancel");
     cancel.setBackground(buttonbg);
     cancel.setForeground(buttonfg);
     cancel.setFont(new Font("Helvetica", Font.BOLD, 14));
     cancel.addActionListener(this);
     addComp(cancel,9,3,1,3,new Insets(-175,1,1,1));

     swin = new StatusWin("SearchForm"); // add the status window component
     addComp(swin,0,402,0,0,new Insets(-80,5,-465,3));

     //show instruction for Search in the status window
     dispSearchInstruction();
}

/**
 * paint method called by the browser to draw a separator line.
 */

 public void paint(Graphics g) {
     g.drawLine(0,160,500,160);
 }

 /**
 * Method that actually sets the parameters and adds the component
 * to the panel. Gets the coordinates, width and height and insets
 * as well as the component object as arguments.
 */

 private void addComp
            ( Component c, int x, int y, int ht, int wt, Insets insets ) {
     gbc.gridx = x;
     gbc.gridy = y;

     gbc.gridwidth = wt;
     gbc.gridheight = ht;

     gbc.insets = insets;

     gb.setConstraints(c,gbc);
     add(c);
 }

/**
 * This method returns a Label object with the text given and with the
 * appropriate colors and fonts.
 */

 private Label makeLabel(String str) {
     Label l = new Label(str);
     l.setForeground(fg);
     l.setFont(new Font("Helvetica", Font.BOLD, 11));
     return l;
 }

/**
 * This is the ActionListener implementation. This method is called when the
 * List button in this panel is clicked. It starts the contacting the server process.
 */

 public void actionPerformed(ActionEvent e) {

     String deptcoursesection; 
     Vector vec; //for accepting returned records
  
     Button b = (Button) e.getSource(); //get the button object 

     String tmpstr = b.getLabel().trim(); // remove any extra spaces and return the
										  // name of the button
     if(tmpstr.equals("Submit")){ 
         if((course.getText().trim()).length() > 0) {
     		try {
                System.out.println("SearchForm->actionPerformed: Submit button pressed.");  
                clearStatusWin();  //clear the status window

                deptcoursesection = getValues();  //collect textfield message
				System.out.println("SearchForm: dept/course/section is "+deptcoursesection);

                if (deptcoursesection.equals("")) {
					swin.writeMsg("No value in the Textfield.");
					System.out.println("SearchForm: No value in the Textfield.");
			    } 
		 
                vec = CourseApplet.obj.search(deptcoursesection); //remote method for SEARCH
                if (vec.size() <= 0) {
					System.out.println("SearchForm->actionPerformed: No record matched in the database file.");
                 } 
			displayData(vec);   
	 		
			}catch (Exception ex) {
	    		System.out.println("SearchForm->actionPerformed: Exception in SEARCH : " + ex);	
				ex.printStackTrace();
			 }
		  }
     }else if(tmpstr.equals("Cancel")) { //check if it is cancel
           clearAll();
     }
     b = null;
     tmpstr = null;
 }


/** 
 * Instruction for Search in the status window
 */

  public void dispSearchInstruction() { 
     swin.writeMsg("\n                * SAMPLE SEARCH *\n");
     swin.writeMsg("   Fill in Dept/Course/Section with 'CIS/675'");
     swin.writeMsg("   or");
     swin.writeMsg("   Fill in Dept/Course/Section with '*' to LIST ALL RECORDS");
     swin.writeMsg("   Press 'Submit' button to start SEARCH");
     swin.writeMsg("   Press 'Cancel' button to CLEAR textfield and status window");
  }




/**
 * This method is used to format and display returned records in the Status Window. 
 */
  private void displayData(Vector vec) {

       Hashtable ht;
       String recordnum;

       recordnum = Integer.toString(vec.size());
     
	   swin.writeMsg("\n                 * "+recordnum+"  RECORD(S) FOUND *");
       System.out.println("\n              * "+recordnum+"  RECORD(S) FOUND *");
       for(Enumeration e = vec.elements(); e.hasMoreElements(); ){
                ht = (Hashtable) e.nextElement();
				displayHashtableInJavaConsole(ht);
                displayHashtable(ht);
        }
   }



/**
 * This method is used to display hashtable in Java Console. 
 */

  private void displayHashtableInJavaConsole(Hashtable ht) {
        System.out.println("\n=======================================================\n");
        System.out.println(" DEPT/COURSE/SECTION  : "+get("course", ht));
        System.out.println(" REFERENCE NO.        : "+get("refno", ht));
        System.out.println(" COURSE NAME          : "+get("cname", ht));
        System.out.println(" INSTRUCTOR           : "+get("instr", ht));
        System.out.println(" LOCATION             : "+get("location", ht));
        System.out.println(" DAY                  : "+get("day", ht));
        System.out.println(" TIME                 : "+get("time", ht));
    }


/**
 * This method is used to display hashtable in the Status Window. 
 */

  private void displayHashtable(Hashtable ht) {

        swin.writeMsg("\n=======================================================\n");
        swin.writeMsg(" DEPT/COURSE/SECTION  : "+get("course", ht));
        swin.writeMsg(" REFERENCE NO.        : "+get("refno", ht));
        swin.writeMsg(" COURSE NAME          : "+get("cname", ht));
        swin.writeMsg(" INSTRUCTOR           : "+get("instr", ht));
        swin.writeMsg(" LOCATION             : "+get("location", ht));
        swin.writeMsg(" DAY                  : "+get("day", ht));
        swin.writeMsg(" TIME                 : "+get("time", ht));
    }


/**
 * This method is used to check keyword field, subsitute null field with " "
 */
 private String get(String key, Hashtable ht){ 
        if(ht.get(key) != null)
            return ht.get(key).toString();
        else
            return " ";
    }


/**
 * This method is used to get values from the 'dept/course/section' textfield. 
 */

 private String getValues() {
     StringBuffer sb = new StringBuffer();
	 sb.append((course.getText()).trim());
     return sb.toString();
 }


/**
 * This method is called from the parent class for clearing or setting the default
 * options and text for the user interface 
 */

 public void clearAll() {
     setDefault();
     clearStatusWin(); 
 }


/**
 * This method clears all the text boxes and sets default values for the choice lists
 * The method can be expanded when textfields other than course have been added to 'Search'   
 */

 private void setDefault(){ 
       course.setText("");
 }



/**
 * This method clears the status window
 */

 private void clearStatusWin() {
     swin.setText(" ");
 }


} // end of class
